# -*- coding: utf-8 -*-
"""
@Time: 2024-01-23 20:30
@Auth: xjjxhxgg
@File: __init__.py
@IDE: PyCharm
@Motto: xhxgg
"""
from .format import coco_to_yolo, yolo_to_coco
from .visualize import visualize

compenents = {
    'coco_to_yolo': coco_to_yolo,
    'yolo_to_coco': yolo_to_coco,
    'visualize': visualize
}



# -*- coding: utf-8 -*-
"""
@Time: 2024-01-30 13:32
@Auth: xjjxhxgg
@File: tiny_copy.py
@IDE: PyCharm
@Motto: xhxgg
"""
import math
import os
import random
from typing import Iterable
import numpy as np
from pycocotools.coco import COCO
import cv2
import json


'''
for copy tiny targets several times to make the dataset more balanced
1. copy the targets to other places in the same image and copy the labels
2. copy the entire image and copy the labels, that is to say, the image is copied several times
    and put into a new image which is of the size x × y 
    where x is the times of copied images in the vertical direction
    and y is the times of copied images in the horizontal direction 
'''


def target_copy(img_path, ann_path, save_path, times=2, parser: callable = None,
                data_format: str = 'yolo', new_name=None, copy_type='tgt', new_img_size=None):
    """
    copy the targets to other places in the same image and copy the labels
    :param img_path: the path of image
    :param ann_path: the path of annotation file
    :param save_path: the directory to save the new image and annotation file
    :param times: the times of copied targets. If copy_type == 'img', times, which is a Union[list,tuple] for [x,y] or number for y=x=sqrt(times), is the times of copied images
        while if copy_type == 'tgt', times,which is a single number, is the times of copied targets, and if there are n targets in the image,
        each target will be copied (times/n) times, if times/n is less than 1, each target will be copied for times times.
    :param parser: if ann_path is xml, parser is required, which is a callable function and returns a list of bbox
     like [{'name':name, 'bbox':[xmin,ymin,xmax,ymax]},...].
     A simple parser is provided in format.py
    :param data_format: the format of annotation file which should be in ['custom', 'yolo', 'coco'].
        If format is 'custom', a  parser is needed which returns the same format as above.
    :param new_name: the new name of the new image. If None, the new name will be the same as the old one.
        It can None if save_path is not a dir.
    :param copy_type: It must be in ['tgt','img'] which relatively mean :
        1. copy the targets to other places in the same image and copy the labels
        2. copy the entire image and copy the labels, that is to say, the image is copied several times
            and put into a new image which is of the size x × y
            where x is the times of copied images in the vertical direction
            and y is the times of copied images in the horizontal direction
    :param new_img_size: Only used when copy_type is 'img'.
        The size of the new image, which is a number, the factor of the old image size, like -2,-1, 1, 2, 3, 4, ...
        where -2 means 1/2 of the old image size, 2 means 2 times of the old image size.
        If None, the new image size will not be changed after copying,
        that is if [x,y]=[2,3] and original img size is [w,h] then the new img is of size [2w,3h].
        If times is a number, the new image size will be [w*sqrt(times),h*sqrt(times)],
        and if new_img_size is a -sqrt(times), the new image size will be the same as the old one.

    :return:
    """
    old_img_name = os.path.basename(img_path)
    old_img = cv2.imread(img_path)
    old_img_height = old_img.shape[0]
    old_img_width = old_img.shape[1]
    if not os.path.exists(save_path):
        os.makedirs(save_path)
    if new_name is None:
        new_name = old_img_name
    save_img_path = os.path.join(save_path, new_name)
    save_ann_path = os.path.join(save_path, new_name.split('.')[0] + '.txt')

    img = old_img.copy()

    random_gen = random.Random()
    random_gen.seed(0)

    if copy_type == 'tgt':
        if data_format == 'yolo':
            bboxes = parser(ann_path, old_img_width, old_img_height)
            if len(bboxes) == 0:
                raise Exception('No bbox found in the annotation file')
            target_num = len(bboxes)
            each_target_times = times // target_num
            if each_target_times == 0:
                each_target_times = times
            targets_to_copy = []
            for i in range(times):
                for bbox in bboxes:
                    name = bbox['name']
                    bbox = bbox['bbox']
                    xmin, ymin, xmax, ymax = bbox
                    target = old_img[ymin:ymax, xmin:xmax, :]
                    for j in range(each_target_times):
                        targets_to_copy.append({'name': name, 'target': target})
            for target in targets_to_copy:
                name = target['name']
                target = target['target']
                # random position to put the target in the image
                # and copy the label to the new annotation file and save the new image
                new_x_min = random_gen.randint(0, old_img_width - target.shape[1])
                new_y_min = random_gen.randint(0, old_img_height - target.shape[0])
                new_x_max = new_x_min + target.shape[1]
                new_y_max = new_y_min + target.shape[0]
                img[new_y_min:new_y_max, new_x_min:new_x_max, :] = target
                new_cx = (new_x_min + new_x_max) / 2
                new_cy = (new_y_min + new_y_max) / 2
                new_w = new_x_max - new_x_min
                new_h = new_y_max - new_y_min
                # new_anns.append([name, new_cx / old_img_width, new_cy / old_img_height, new_w / old_img_width, new_h / old_img_height])
                with open(save_ann_path, 'a') as f:
                    f.write(
                        f'{name} {new_cx / old_img_width} {new_cy / old_img_height} {new_w / old_img_width} {new_h / old_img_height}\n')

            cv2.imwrite(save_img_path, img)


        elif data_format == 'coco':
            save_ann_path = save_ann_path.replace('txt','json')
            pass
        else:
            pass
    elif copy_type == 'img':
        if isinstance(times, int):
            x_times, y_times = int(math.sqrt(times)), int(math.sqrt(times))
        elif isinstance(times, Iterable):
            x_times, y_times = times

        new_img_width = old_img_width * y_times
        new_img_height = old_img_height * x_times
        img = np.ones((new_img_height, new_img_width, 3), dtype=np.uint8) * 255
        for i in range(x_times):
            for j in range(y_times):
                img[i * old_img_height:(i + 1) * old_img_height, j * old_img_width:(j + 1) * old_img_width, :] = old_img
        if new_img_size is not None:
            if new_img_size < 0:
                new_img_size = 1 / -new_img_size
            img = cv2.resize(img, (int(new_img_width * new_img_size), int(new_img_height * new_img_size)))
        cv2.imwrite(save_img_path, img)

        if data_format == 'yolo':
            bboxes = parser(ann_path, old_img_width, old_img_height)
            for i in range(x_times):
                for j in range(y_times):
                    for bbox in bboxes:
                        name = bbox['name']
                        bbox = bbox['bbox']
                        xmin, ymin, xmax, ymax = bbox
                        new_x_min = j * old_img_width + xmin
                        new_y_min = i * old_img_height + ymin
                        new_x_max = j * old_img_width + xmax
                        new_y_max = i * old_img_height + ymax
                        with open(save_ann_path, 'a') as f:
                            f.write(
                                f'{name} {(new_x_min + new_x_max) / 2 / new_img_width} {(new_y_min + new_y_max) / 2 / new_img_height} {(new_x_max - new_x_min) / new_img_width} {(new_y_max - new_y_min) / new_img_height}\n')
        elif data_format == 'coco':
            save_ann_path = save_ann_path.replace('txt', 'json')
            coco = COCO(ann_path)
            imgs = coco.imgs
            img_ids = None
            for img in imgs.values():
                if img['file_name'] == old_img_name:
                    img_ids = img['id']
                    break
            ann_ids = coco.getAnnIds(img_ids)
            anns_ = coco.loadAnns(ann_ids)
            max_ann_id = max(coco.getAnnIds())
            next_ann_id = max_ann_id + 1
            for i in range(x_times):
                for j in range(y_times):
                    for ann_id in ann_ids:
                        ann = coco.anns[ann_id].copy()
                        bbox = ann['bbox']
                        new_x_min = j * old_img_width + bbox[0]
                        new_y_min = i * old_img_height + bbox[1]
                        new_w = bbox[2]
                        new_h = bbox[3]
                        ann['bbox'] = [new_x_min * new_img_size, new_y_min * new_img_size, new_w * new_img_size, new_h * new_img_size]
                        ann['image_id'] = img_ids
                        ann['id'] = next_ann_id
                        ann['area'] = new_w * new_h * new_img_size * new_img_size
                        coco.anns[next_ann_id] = ann
                        next_ann_id += 1

            for ann_id in ann_ids:
                coco.anns.pop(ann_id)
            coco.imgs[img_ids]['width'] = int(new_img_width * new_img_size)
            coco.imgs[img_ids]['height'] = int(new_img_height * new_img_size)
            anns = {'images': list(coco.imgs.values()), 'annotations': list(coco.anns.values()), 'categories': list(coco.cats.values()), 'info': [], 'licenses': []}
            with open(save_ann_path, 'w') as f:
                json.dump(anns, f) # [ann for ann in  anns['annotations'] if ann['id']==5415] max([ann['id'] for ann in  anns['annotations']])

        else:
            pass
    else:
        raise Exception('copy_type must be in [\'tgt\',\'img\']')


if __name__ == '__main__':
    from format import yolo_parser
    from visualize import visualize
    #
    # img_path = r'F:\GD\dataset\coco128\images\train2017\000000000025.jpg'
    # ann_path = r'F:\GD\dataset\coco128\labels\train2017\000000000025.txt'
    # # target_copy(img_path, ann_path, '.', times=2, parser=yolo_parser, data_format='yolo', new_name=None, copy_type='tgt')
    # target_copy(img_path, ann_path, '.', times=4, parser=yolo_parser, data_format='yolo', copy_type='img', new_img_size=-2)
    # visualize('000000000025.jpg', '000000000025.txt')
    # coco = COCO(r'F:\GD\dataset\UAVDT\annotations\M0101.json')
    # js = json.load(open(r'F:\GD\dataset\UAVDT\annotations\M0101.json', 'r'))
    # print()
    # ann_path = r'F:\GD\dataset\HIT-UAV\annotations\train.json'
    # img_path = r'F:\GD\dataset\HIT-UAV\images\train\0_60_30_0_01611.jpg'
    ann_path = r'F:\GD\dataset\UAVDT\annotations\M0101.json'
    img_path = r'F:\GD\dataset\UAVDT\images\M0101-img000008.jpg'
    save_path = r'..'
    target_copy(img_path, ann_path, '..', times=4, data_format='coco', copy_type='img', new_img_size=-2)
    visualize('M0101-img000008.jpg', 'M0101-img000008.json')
    # visualize(img_path, ann_path)


# -*- coding: utf-8 -*-
"""
@Time: 2024-01-29 13:30
@Auth: xjjxhxgg
@File: visualize.py
@IDE: PyCharm
@Motto: xhxgg
"""
from typing import Union

import cv2
import os
import json
import numpy as np
from .format import xml_parser, yolo_parser
from pycocotools.coco import COCO

def visualize(img_path: str, ann_path: str, save_path: str = None, color: tuple = None, parser: callable = None, format: str='yolo') -> None:
    """
    visualize the image with annotations, at present only support object detection with bbox
    :param img_path: the path of image
    :param ann_path: the path of annotation file
    :param save_path: if not None, save the image with bbox to save_path else show the image with bbox
    :param color: the color of bbox or mask
    :param parser: if ann_path is xml, parser is required, which is a callable function and returns a list of bbox
     like [{'name':name, 'bbox':[xmin,ymin,xmax,ymax]},...] or [[xmin,ymin,xmax,ymax],...].
     It takes ann_path as input if the annotations for each image are in a seperate file respectively.,
     or takes img_path and ann_path as input if the annotations for each image are all in one file,
     and the parser should accept img_path as the first input and ann_path the second.
     A simple parser is provided in format.py
    :param format: the format of annotation file which should be in ['custom', 'yolo'].
     If format is 'custom', a  parser is needed which returns the same format as above.
    :return:
    """
    img = None
    if color is None:
        color = (0, 0, 255)
    if ann_path.endswith('.json'):
        if parser is None:  # coco
            filename = os.path.basename(img_path)
            coco = COCO(ann_path)
            imgs = coco.imgs
            img_id = None
            file_name_key = 'file_name' if 'file_name' in imgs[coco.getImgIds()[0]] else 'filename'
            for img in imgs.values():
                if img[file_name_key] == filename:
                    img_id = img['id']
                    break
            img = cv2.imread(img_path)
            if img_id is None:
                raise Exception('Image not found in ann file')
            anns = coco.loadAnns(coco.getAnnIds(imgIds=img_id))  # coco.loadAnns(4635)
            if len(anns) == 0:
                raise Exception('No bbox found in ann file')
            for ann_ann in anns:
                bbox = ann_ann['bbox']
                xmin = (bbox[0])
                ymin = bbox[1]
                w = bbox[2]
                h = bbox[3]
                xmax = xmin + w
                ymax = ymin + h
                xmin = int(xmin)
                ymin = int(ymin)
                xmax = int(xmax)
                ymax = int(ymax)
                cv2.rectangle(img, (xmin, ymin), (xmax, ymax), color, 2)
        else:
            bboxes = parser(img_path,ann_path)
            img = cv2.imread(img_path)
            if len(bboxes) == 0:
                raise Exception('No bbox found in ann file')
            for bbox in bboxes:
                if isinstance(bbox, dict):
                    bbox = bbox['bbox']
                x = bbox[0]
                y = bbox[1]
                w = bbox[2]
                h = bbox[3]
                cv2.rectangle(img, (x, y), (x + w, y + h), color, 2)
    elif ann_path.endswith('.xml'):  # voc
        if parser is None:
            raise Exception('parser is None')
        img = cv2.imread(img_path)
        bboxes = parser(ann_path)
        if len(bboxes) == 0:
            raise Exception('No bbox found in ann file')
        for bbox in bboxes:
            if isinstance(bbox, dict):
                bbox = bbox['bbox']
            x = bbox[0]
            y = bbox[1]
            w = bbox[2]
            h = bbox[3]
            cv2.rectangle(img, (x, y), (x + w, y + h), color, 2)
    elif ann_path.endswith('.txt'):
        if format == 'yolo':  # yolo
            img = cv2.imread(img_path)
            img_h, img_w, _ = img.shape
            bboxes = yolo_parser(ann_path, img_w, img_h)
            if len(bboxes) == 0:
                raise Exception('No bbox found in ann file')
            for bbox in bboxes:
                bbox = bbox['bbox']
                xmin, ymin, xmax, ymax = bbox
                cv2.rectangle(img, (xmin, ymin), (xmax, ymax), color, 2)

        elif format == 'custom':
            if parser is None:
                raise Exception('parser is None')
            img = cv2.imread(img_path)
            bboxes = parser(ann_path)
            if len(bboxes) == 0:
                raise Exception('No bbox found in ann file')
            for bbox in bboxes:
                if isinstance(bbox, dict):
                    bbox = bbox['bbox']
                x = bbox[0]
                y = bbox[1]
                w = bbox[2]
                h = bbox[3]
                cv2.rectangle(img, (x, y), (x + w, y + h), color, 2)
    else:
        raise Exception('Annotation file not supported')
    if img is not None:
        if save_path is not None:
            if os.path.isdir(save_path):
                save_path = os.path.join(save_path, 'vis.jpg')
            cv2.imwrite(save_path, img)
        else:
            cv2.imshow('vis', img)
            cv2.waitKey(0)
    else:
        raise Exception('Image not found')



# -*- coding: utf-8 -*-
"""
@Time: 2024-01-29 14:45
@Auth: xjjxhxgg
@File: train_val_divide.py
@IDE: PyCharm
@Motto: xhxgg
"""
import random
import json
import copy


def coco_divide(coco_json, train_json, val_json, train_ratio=0.8):
    import json
    import random
    coco_json = json.load(open(coco_json, 'r'))
    images = coco_json['images']
    annotations = coco_json['annotations']
    categories = coco_json['categories']
    train_images = []
    val_images = []
    train_annotations = []
    val_annotations = []
    train_categories = []
    val_categories = []
    train_img_ids = []
    val_img_ids = []
    for img in images:
        if random.random() < train_ratio:
            train_images.append(img)
            train_img_ids.append(img['id'])
        else:
            val_images.append(img)
            val_img_ids.append(img['id'])
    for ann in annotations:
        if ann['image_id'] in train_img_ids:
            train_annotations.append(ann)
        else:
            val_annotations.append(ann)
    for cat in categories:
        if cat['id'] in [ann['category_id'] for ann in train_annotations]:
            train_categories.append(cat)
        else:
            val_categories.append(cat)
    train = {
        'images': train_images,
        'annotations': train_annotations,
        'categories': train_categories,
        'info': [],
        'license': []
    }
    val = {
        'images': val_images,
        'annotations': val_annotations,
        'categories': val_categories,
        'info': [],
        'license': []
    }
    json.dump(train, open(train_json, 'w'))
    json.dump(val, open(val_json, 'w'))


def yolo_divide(total_lbl_root, total_img_root, yolo_root, train_ratio=0.8):
    import os
    import shutil
    for root, dirs, files in os.walk(total_lbl_root):
        for file in files:
            if file.endswith('.txt'):
                if random.random() < train_ratio:
                    shutil.copy(str(os.path.join(total_lbl_root, file)),
                                str(os.path.join(yolo_root, 'labels', 'train', file)))
                    shutil.copy(str(os.path.join(total_img_root, file.replace('.txt', '.jpg'))),
                                str(os.path.join(yolo_root, 'images', 'train', file.replace('.txt', '.jpg'))))
                else:
                    shutil.copy(str(os.path.join(total_lbl_root, file)),
                                str(os.path.join(yolo_root, 'labels', 'val', file)))
                    shutil.copy(str(os.path.join(total_img_root, file.replace('.txt', '.jpg'))),
                                str(os.path.join(yolo_root, 'images', 'val', file.replace('.txt', '.jpg'))))


def coco_merge(*args):
    """
    Merge multiple coco json files into one. All the json files should have the totally same categories.
    :param args:
    :return:
    """
    coco_json1 = args[0]
    coco_json2 = args[1]
    json_cnt = 2
    json1 = json.load(open(coco_json1, 'r'))
    json2 = json.load(open(coco_json2, 'r'))
    img_id = 0
    ann_id = 0
    merged_json, img_id, ann_id = __merge(json1, json2, img_id, ann_id)
    while json_cnt < len(args):
        new_json = json.load(open(args[json_cnt], 'r'))
        merged_json, img_id, ann_id = __merge(merged_json, new_json, img_id, ann_id)
        json_cnt += 1
    return merged_json


def __merge(json1, json2, img_id, ann_id):
    merged_json = {'images': [], 'annotations': [], 'categories': copy.deepcopy(json1['categories']),
                   'linceses': copy.deepcopy(json1['linceses']), 'info': copy.deepcopy(json1['info'])}
    images1 = json1['images']
    images2 = json2['images']
    annotations1 = json1['annotations']
    annotations2 = json2['annotations']
    img_id_map = {1: {}, 2: {}}
    ann_id_map = {1: {}, 2: {}}
    for img in images1:
        img_id = max(img_id, img['id'])
    img_id += 1
    for ann in annotations1:
        ann_id = max(ann_id, ann['id'])
    ann_id += 1
    for img in images1:
        new_img = img.copy()
        new_img['id'] = img_id
        img_id_map[1][img['id']] = img_id
        img_id += 1
        merged_json['images'].append(new_img)
    for img in images2:
        new_img = img.copy()
        new_img['id'] = img_id
        img_id_map[2][img['id']] = img_id
        img_id += 1
        merged_json['images'].append(new_img)
    for ann in annotations1:
        new_ann = ann.copy()
        new_ann['id'] = ann_id
        new_ann['image_id'] = img_id_map[1][ann['image_id']]
        ann_id_map[1][ann['id']] = ann_id
        ann_id += 1
        merged_json['annotations'].append(new_ann)
    for ann in annotations2:
        new_ann = ann.copy()
        new_ann['id'] = ann_id
        new_ann['image_id'] = img_id_map[2][ann['image_id']]
        ann_id_map[2][ann['id']] = ann_id
        ann_id += 1
        merged_json['annotations'].append(new_ann)

    return merged_json, img_id, ann_id



# -*- coding: utf-8 -*-
"""
@Time: 2024-01-23 20:20
@Auth: xjjxhxgg
@File: coco_to_yolo.py
@IDE: PyCharm
@Motto: xhxgg
"""

import json
import os

import cv2


def coco_to_yolo(json_path, yolo_root, cat_reid=False):
    ann_file = json.load(open(json_path, 'r'))
    images = ann_file['images']
    annotations = ann_file['annotations']
    categories = ann_file['categories']
    categories_reid = [cat['id'] for cat in categories]
    categories_reid.sort()
    img_ann_map = {}
    for img in images:
        img_ann_map[img['id']] = []
    for ann in annotations:
        img_ann_map[ann['image_id']].append(ann)
    file_name_key = 'file_name' if 'file_name' in images[0] else 'filename'
    for img in images:
        img_name = img[file_name_key]
        img_width = img['width']
        img_height = img['height']
        img_id = img['id']
        img_anns = img_ann_map[img_id]
        yolo_txt_path = os.path.join(yolo_root, img_name.split('.')[0] + '.txt')
        with open(yolo_txt_path, 'w') as f:
            for ann in img_anns:
                cat_id = ann['category_id']
                if cat_reid:
                    cat_id = categories_reid.index(cat_id)
                bbox = ann['bbox']
                x = bbox[0]
                y = bbox[1]
                w = bbox[2]
                h = bbox[3]
                x_center = x + w / 2
                y_center = y + h / 2
                x_center /= img_width
                y_center /= img_height
                w /= img_width
                h /= img_height
                f.write(str(cat_id) + ' ' + str(x_center) + ' ' + str(y_center) + ' ' + str(w) + ' ' + str(h) + '\n')


def yolo_to_coco(yolo_root, json_path, categories, img_root, img_id=0, ann_id=0, info='', licenses=''):
    ann_file = open(json_path, 'w')
    images = []
    annotations = []
    for root, dirs, files in os.walk(yolo_root):
        for file in files:
            if file.endswith('.txt'):
                img_id += 1
                img_name = file.split('.')[0] + '.jpg'
                img_file = os.path.join(img_root, img_name)
                img = cv2.imread(img_file)
                height, width, _ = img.shape
                img = {'file_name': img_name, 'height': height, 'width': width, 'id': img_id}
                images.append(img)
                txt_path = os.path.join(root, file)
                with open(txt_path, 'r') as f:
                    for line in f.readlines():
                        line = line.strip()
                        if line == '':
                            continue
                        cat, x, y, w, h = line.split(' ')
                        x = float(x)
                        y = float(y)
                        w = float(w)
                        h = float(h)
                        x *= width
                        y *= height
                        w *= width
                        h *= height
                        xmin = int(x - w / 2)
                        ymin = int(y - h / 2)
                        cat = int(cat)
                        bbox = [xmin, ymin, w, h]
                        ann_id += 1
                        ann = {'id': ann_id, 'image_id': img_id, 'category_id': cat, 'bbox': bbox}
                        annotations.append(ann)
    ann_file.write(json.dumps(
        {'images': images, 'annotations': annotations, 'categories': categories, 'info': info, 'licenses': licenses}))
    ann_file.close()


def coco_parser(ann_path):
    '''
    return {'images': images, 'categories': categories, 'img_ann_map': img_ann_map}
    :param ann_path:
    :return:
    '''
    ann = json.load(open(ann_path, 'r'))
    images = ann['images']
    annotations = ann['annotations']
    categories = ann['categories']
    img_ann_map = {}
    for img in images:
        img_id = img['id']
        img_ann_map[img_id] = []
        for ann in annotations:
            if ann['image_id'] == img_id:
                img_ann_map[img_id].append(ann)

    return {'images': images, 'categories': categories, 'img_ann_map': img_ann_map}


def yolo_parser(yolo_path, img_w=None, img_h=None):
    """
    return bboxes [{'name':name,'bbox':[xmin,ymin,xmax,ymax]},...] or [{'name':name,'bbox':[cx,cy,w,h]},...]
    :param yolo_path:
    :param img_w: image width
    :param img_h: image height
    if img_w and img_h are not None, the bbox will be scaled to the original image size,
    otherwise the bbox is the relative size
    :return:
    """
    bboxes = []
    with open(yolo_path, 'r') as f:
        for line in f.readlines():
            line = line.strip()
            if line == '':
                continue
            cat, x, y, w, h = line.split(' ')
            cat = int(cat)
            cx = float(x)
            cy = float(y)
            w = float(w)
            h = float(h)
            if img_w and img_h:
                xmin = int((cx - w / 2) * img_w)
                ymin = int((cy - h / 2) * img_h)
                xmax = int((cx + w / 2) * img_w)
                ymax = int((cy + h / 2) * img_h)
                bboxes.append({'name': cat, 'bbox': [xmin, ymin, xmax, ymax]})
            else:
                bboxes.append({'name': cat, 'bbox': [cx, cy, w, h]})
    return bboxes


def xml_parser(ann_path):
    '''
    return bboxes [{'name':name,'bbox':[xmin,ymin,xmax,ymax]},...]
    :param ann_path:
    :return:
    '''
    from xml.dom import minidom
    tree = minidom.parse(ann_path)
    root = tree.documentElement
    objects = root.getElementsByTagName('object')
    bboxes = []
    for object in objects:
        name = object.getElementsByTagName('name')[0].childNodes[0].data
        bndboxes = object.getElementsByTagName('bndbox')
        for bndbox in bndboxes:
            xmin = int(bndbox.getElementsByTagName('xmin')[0].childNodes[0].data)
            ymin = int(bndbox.getElementsByTagName('ymin')[0].childNodes[0].data)
            xmax = int(bndbox.getElementsByTagName('xmax')[0].childNodes[0].data)
            ymax = int(bndbox.getElementsByTagName('ymax')[0].childNodes[0].data)
            bboxes.append({'name': name, 'bbox': [xmin, ymin, xmax, ymax]})
    return bboxes



# -*- encoding: utf-8 -*-
"""
@Time: 2024-03-11 19:26
@Auth: xjjxhxgg
@File: cal_tgt_size_heatmap.py
@IDE: PyCharm
@Motto: xhxgg
"""
import os
from typing import Union

import cv2
from format import yolo_parser, coco_parser
from matplotlib import pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from collections import Counter
import numpy as np
import seaborn as sb
import pandas as pd


def yolo_cal_tgt_size_dist(yolo_root=r'/data/ll/code/sod/YOLOv8_With_ODConv/dataset/3x3',s=None):
    def __get_sizes(imgs, img_root, lbl_root):
        size_cnt_map = {}
        sizes = []
        hw_pair = []
        for img in imgs:
            img_path = fr'{img_root}/{img}'
            lbl_path = fr'{lbl_root}/{img.replace("jpg", "txt")}'
            img = cv2.imread(img_path)
            h, w, _ = img.shape
            boxes = yolo_parser(lbl_path)
            for nb in boxes:
                box = nb['bbox']
                size = int(box[2] * box[3] * w * h)
                if size not in size_cnt_map:
                    size_cnt_map[size] = 0
                size_cnt_map[size] += 1
                sizes.append(size)
                bw = int(box[2] * w)
                bh = int(box[3] * h)
                hw_pair.append((bh, bw))
        return sizes, size_cnt_map, hw_pair

    has_train = 'train' in os.listdir(f'{yolo_root}/images') and 'train' in os.listdir(f'{yolo_root}/labels')
    has_test = 'test' in os.listdir(f'{yolo_root}/images') and 'test' in os.listdir(f'{yolo_root}/labels')
    has_val = 'val' in os.listdir(f'{yolo_root}/images') and 'val' in os.listdir(f'{yolo_root}/labels')
    if not has_train and not has_test and not has_val:
        raise ValueError('No train or test set found')
    if s is not None:
        img_root = fr'{yolo_root}/images/{s}'
        imgs = os.listdir(img_root)
        lbl_root = fr'{yolo_root}/labels/{s}'
        sizes, size_cnt_map, hw_pair = __get_sizes(imgs, img_root, lbl_root)
    elif not has_train and not has_val:
        img_root = fr'{yolo_root}/images/test'
        imgs = os.listdir(img_root)
        lbl_root = fr'{yolo_root}/labels/test'
        sizes, size_cnt_map, hw_pair = __get_sizes(imgs, img_root, lbl_root)
    elif not has_test and not has_val:
        img_root = fr'{yolo_root}/images/train'
        imgs = os.listdir(img_root)
        lbl_root = fr'{yolo_root}/labels/train'
        sizes, size_cnt_map, hw_pair = __get_sizes(imgs, img_root, lbl_root)
    elif not has_test and not has_train:
        img_root = fr'{yolo_root}/images/val'
        imgs = os.listdir(img_root)
        lbl_root = fr'{yolo_root}/labels/val'
        sizes, size_cnt_map, hw_pair = __get_sizes(imgs, img_root, lbl_root)
    elif not has_val:
        img_root = fr'{yolo_root}/images/train'
        imgs = os.listdir(img_root)
        lbl_root = fr'{yolo_root}/labels/train'
        sizes_train, size_cnt_map_train, hw_pair_train = __get_sizes(imgs, img_root, lbl_root)
        img_root = fr'{yolo_root}/images/test'
        imgs = os.listdir(img_root)
        lbl_root = fr'{yolo_root}/labels/test'
        sizes_test, size_cnt_map_test, hw_pair_test = __get_sizes(imgs, img_root, lbl_root)
        sizes = sizes_train + sizes_test
        size_cnt_map = size_cnt_map_train
        for k, v in size_cnt_map_test.items():
            if k not in size_cnt_map:
                size_cnt_map[k] = 0
            size_cnt_map[k] += v
        hw_pair = hw_pair_train + hw_pair_test
    elif not has_test:
        img_root = fr'{yolo_root}/images/train'
        imgs = os.listdir(img_root)
        lbl_root = fr'{yolo_root}/labels/train'
        sizes_train, size_cnt_map_train, hw_pair_train = __get_sizes(imgs, img_root, lbl_root)
        img_root = fr'{yolo_root}/images/val'
        imgs = os.listdir(img_root)
        lbl_root = fr'{yolo_root}/labels/val'
        sizes_val, size_cnt_map_val, hw_pair_val = __get_sizes(imgs, img_root, lbl_root)
        sizes = sizes_train + sizes_val
        size_cnt_map = size_cnt_map_train
        for k, v in size_cnt_map_val.items():
            if k not in size_cnt_map:
                size_cnt_map[k] = 0
            size_cnt_map[k] += v
        hw_pair = hw_pair_train + hw_pair_val
    elif not has_train:
        img_root = fr'{yolo_root}/images/test'
        imgs = os.listdir(img_root)
        lbl_root = fr'{yolo_root}/labels/test'
        sizes_test, size_cnt_map_test, hw_pair_test = __get_sizes(imgs, img_root, lbl_root)
        img_root = fr'{yolo_root}/images/val'
        imgs = os.listdir(img_root)
        lbl_root = fr'{yolo_root}/labels/val'
        sizes_val, size_cnt_map_val, hw_pair_val = __get_sizes(imgs, img_root, lbl_root)
        sizes = sizes_test + sizes_val
        size_cnt_map = size_cnt_map_test
        for k, v in size_cnt_map_val.items():
            if k not in size_cnt_map:
                size_cnt_map[k] = 0
            size_cnt_map[k] += v
        hw_pair = hw_pair_test + hw_pair_val
    else:
        img_root = fr'{yolo_root}/images/train'
        imgs = os.listdir(img_root)
        lbl_root = fr'{yolo_root}/labels/train'
        sizes_train, size_cnt_map_train, hw_pair_train = __get_sizes(imgs, img_root, lbl_root)
        img_root = fr'{yolo_root}/images/test'
        imgs = os.listdir(img_root)
        lbl_root = fr'{yolo_root}/labels/test'
        sizes_test, size_cnt_map_test, hw_pair_test = __get_sizes(imgs, img_root, lbl_root)
        img_root = fr'{yolo_root}/images/val'
        imgs = os.listdir(img_root)
        lbl_root = fr'{yolo_root}/labels/val'
        sizes_val, size_cnt_map_val, hw_pair_val = __get_sizes(imgs, img_root, lbl_root)
        sizes = sizes_train + sizes_test + sizes_val
        size_cnt_map = size_cnt_map_train
        for k, v in size_cnt_map_test.items():
            if k not in size_cnt_map:
                size_cnt_map[k] = 0
            size_cnt_map[k] += v
        for k, v in size_cnt_map_val.items():
            if k not in size_cnt_map:
                size_cnt_map[k] = 0
            size_cnt_map[k] += v
        hw_pair = hw_pair_train + hw_pair_test + hw_pair_val
    return sizes, size_cnt_map, hw_pair




def cal_tgt_size_dist(*,res='c', yolo_root=None, coco_root=None, format='yolo',s=None):
    """
    :param res: If res is set to 'c', return the size_cnt_map,
    else return the sizes list, size cnt map, hieght width pair
    :param yolo_root:
    :param coco_root:
    :param format:
    :return:
    """
    if format == 'yolo':
        sizes, size_cnt_map, hw_pair = yolo_cal_tgt_size_dist(yolo_root,s)
    elif format == 'coco':
        raise NotImplementedError
    if res == 'c':
        return size_cnt_map
    return sizes, size_cnt_map, hw_pair


def gen_heatmap(*, res='a', base=None,yolo_root=None, coco_root=None,
                filename='heatmap.png', threshold: Union[list, tuple, int, float] = 100,s=None):
    """
    :param base: If base is not None, the width and height will be rounded
    to the maximum multiple of base less than the original width and height
    :param filename:
    :param threshold: The maximum width and height to be shown in the heatmap
    :return:
    """

    format = 'yolo'
    if yolo_root is None and coco_root is None:
        raise ValueError('yolo_root and coco_root cannot be both None')
    if yolo_root is not None and coco_root is not None:
        raise ValueError('yolo_root and coco_root cannot be both not None')
    if yolo_root is not None:
        _, _, hw_pair = cal_tgt_size_dist(res=res,yolo_root=yolo_root, format=format,s=s)
    else:
        _, _, hw_pair = cal_tgt_size_dist(coco_root=coco_root, format=format,s=s)
    # hw_pair = hw_pair[:100]
    h_threshold = 100
    w_threshold = 100
    if isinstance(threshold, (list, tuple)):
        h_threshold = threshold[0]
        w_threshold = threshold[1]
    elif isinstance(threshold, (int, float)):
        h_threshold = threshold
        w_threshold = threshold
    hs = []
    ws = []
    if base is not None:
        for i in range(len(hw_pair)):
            x = hw_pair[i][0] // base * base
            y = hw_pair[i][1] // base * base
            hw_pair[i] = (x, y)
    fig = plt.figure()
    counter = dict(Counter(hw_pair))
    for k, v in counter.items():
        if k[0] > h_threshold or k[1] > w_threshold:
            continue
        hs.append(k[0])
        ws.append(k[1])
    heat_map_data = []
    for i in range(100):
        heat_map_data.append([])
        for j in range(100):
            if (i, j) in counter:
                heat_map_data[i].append(counter[(i, j)])
            else:
                heat_map_data[i].append(0)
    heat_map_data = pd.DataFrame(heat_map_data)
    sb.heatmap(heat_map_data)
    plt.xlabel('height')
    plt.ylabel('width')
    plt.savefig(filename)



# -*- encoding: utf-8 -*-
"""
@Time: 2024-03-11 18:46
@Auth: xjjxhxgg
@File: setup.py
@IDE: PyCharm
@Motto: xhxgg
"""
from setuptools import setup, find_packages

setup(
    name='pyzyj',
    version='0.4',
    packages=find_packages(),
    install_requires=[
        # list your project dependencies here
    ],
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
)
